@@ -1,22 +1,29 @@
 # -*- coding: utf8 -*-
 import numpy as np
 import PDielec.Calculator as Calculator
-from PyQt5.QtWidgets  import  QWidget, QApplication
-from PyQt5.QtWidgets  import  QComboBox, QLabel
-from PyQt5.QtWidgets  import  QCheckBox
-from PyQt5.QtWidgets  import  QVBoxLayout, QFormLayout
-from PyQt5.QtWidgets  import  QDoubleSpinBox, QTableWidget, QTableWidgetItem
-from PyQt5.QtWidgets  import  QSizePolicy
-from PyQt5.QtCore     import  Qt, QSize, QCoreApplication
-from PDielec.Constants import  wavenumber, amu, PI, angstrom
-from PDielec.Constants import  average_masses, isotope_masses
-from PDielec.Utilities import  Debug
+from PyQt5.QtWidgets            import  QWidget, QApplication
+from PyQt5.QtWidgets            import  QComboBox, QLabel
+from PyQt5.QtWidgets            import  QCheckBox
+from PyQt5.QtGui                import  QFont
+from PyQt5.QtWidgets            import  QVBoxLayout, QFormLayout
+from PyQt5.QtWidgets            import  QDoubleSpinBox, QTableWidget, QTableWidgetItem
+from PyQt5.QtWidgets            import  QSizePolicy
+from PyQt5.QtCore               import  Qt, QSize, QCoreApplication
+from PDielec.Constants          import  wavenumber, amu, PI, angstrom
+from PDielec.Constants          import  average_masses, isotope_masses
+from PDielec.Utilities          import  Debug
+from PDielec.DielectricFunction import  DielectricFunction
+from functools                  import  partial
 
 class FixedQTableWidget(QTableWidget):
     def __init__(self, *args, parent=None, rows=None, columns=None):
         self.columns = columns
         self.rows = rows
         super(QTableWidget, self).__init__(*args)
+        # fnt = QFont()
+        # fnt.setPointSize(12)
+        # fnt.setFamily("Arial")
+        # self.setFont(fnt)
 
     def sizeHint(self):
         width = 0
@@ -46,8 +53,10 @@ class SettingsTab(QWidget):
         super(QWidget, self).__init__(parent)
         global debugger
         debugger = Debug(debug, 'SettingsTab:')
+        debugger.print('Start:: initialisation')
         self.notebook = parent
-        self.dirty = True
+        self.refreshRequired = True
+        self.calculationRequired = False
         self.settings = {}
         self.settings['Eckart flag'] = True
         self.settings['Neutral Born charges'] = False
@@ -63,6 +72,9 @@ class SettingsTab(QWidget):
         self.sigmas_cm1 = []
         self.oscillator_strengths = []
         self.mass_weighted_normal_modes = None
+        self.CrystalPermittivity = None
+        self.vs_cm1 = []
+        self.crystal_permittivity = []
         # get the reader from the main tab
         self.reader = self.notebook.reader
         # Create second tab - SettingsTab
@@ -115,7 +127,7 @@ class SettingsTab(QWidget):
         self.mass_cb.setCurrentIndex(0)
         form.addRow(QLabel('Atomic mass defintion:', self), self.mass_cb)
         # Create Table containing the masses - block signals until the table is loaded
-        self.element_masses_tw = FixedQTableWidget(self)
+        self.element_masses_tw = FixedQTableWidget(parent=self)
         self.element_masses_tw.setToolTip('Individual element masses can be modified here')
         self.element_masses_tw.itemClicked.connect(self.on_element_masses_tw_itemClicked)
         self.element_masses_tw.itemChanged.connect(self.on_element_masses_tw_itemChanged)
@@ -140,7 +152,7 @@ class SettingsTab(QWidget):
         #
         # Create the Optical permittivity table widget and block signals until a click on the widget
         #
-        self.optical_tw = FixedQTableWidget(3,3)
+        self.optical_tw = FixedQTableWidget(3,3,parent=self)
         self.optical_tw.setToolTip('The optical permittivity is taken from the calculation where this is possible.  If it is not availble suitbale values should be provided here')
         # Set the header names
         self.optical_tw.setHorizontalHeaderLabels(['x','y','z'])
@@ -155,7 +167,7 @@ class SettingsTab(QWidget):
         vbox.addLayout(form)
         # output window
         # Create Table containing the IR active modes
-        self.output_tw = FixedQTableWidget(self)
+        self.output_tw = FixedQTableWidget(parent=self)
         self.output_tw.setToolTip('Output showing the frequencies and strengths of the IR active modes only')
         self.output_tw.itemChanged.connect(self.on_output_tw_itemChanged)
         self.output_tw.setRowCount(1)
@@ -164,26 +176,25 @@ class SettingsTab(QWidget):
         # finalise the layout
         self.setLayout(vbox)
         QCoreApplication.processEvents()
+        debugger.print('Finished:: initialisation')
 
     def setElementMass(self,element,mass):
+        debugger.print('Start::  setElementMass',element,mass)
         self.settings['Mass definition'] = 'gui'
         self.masses_dict[element] = mass
         self.mass_cb.setCurrentIndex(3)
         self.set_masses_tw()
-        self.calculateButtonClicked()
+        self.refreshRequired = True
+        self.refresh()
+        debugger.print('Finished::  setElementMass',element,mass)
 
-    def calculateButtonClicked(self):
-        debugger.print('Button 1 pressed')
+    def createIntensityTable(self):
+        debugger.print('Start:: createIntensityTable')
         self.reader = self.notebook.reader
         # Only calculate if the reader is set
         if self.reader is None:
+            debugger.print('createIntensityTable aborting as now reader available')
             return
-        QApplication.setOverrideCursor(Qt.WaitCursor)
-        # Flag the fact that we need new calculations and new analysis
-        self.notebook.plottingCalculationRequired = True
-        self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
-        #self.reader.read_output()
         if self.settings['Neutral Born charges']:
             self.reader.neutralise_born_charges()
         else:
@@ -205,9 +216,6 @@ class SettingsTab(QWidget):
         self.mass_weighted_normal_modes = self.reader.calculate_mass_weighted_normal_modes()
         # convert cm-1 to au
         self.frequencies_cm1 = self.reader.frequencies
-        # work out the degeneraceies
-        self.degenerate_lists = {}
-#        #
         frequencies = np.array(self.frequencies_cm1) * wavenumber
         if len(self.sigmas_cm1) == 0:
             self.sigmas_cm1 = [ self.settings['Sigma value'] for i in self.frequencies_cm1 ]
@@ -233,9 +241,30 @@ class SettingsTab(QWidget):
             else:
                 self.modes_selected.append(False)
         # Calculate the ionic contribution to the permittivity
-        frequencies = wavenumber*np.array(self.frequencies_cm1)
-        volume = self.reader.volume*angstrom*angstrom*angstrom
-        self.epsilon_ionic = Calculator.ionic_permittivity(mode_list, self.oscillator_strengths, frequencies, volume )
+        frequencies_au = wavenumber*np.array(self.frequencies_cm1)
+        volume_au = self.reader.volume*angstrom*angstrom*angstrom
+        self.epsilon_ionic = Calculator.ionic_permittivity(mode_list, self.oscillator_strengths, frequencies_au, volume_au )
+        # Make an np array of epsilon infinity
+        epsilon_inf = np.array(self.settings['Optical permittivity'])
+        #
+        # If the reader already has a Dielectric Constant then use this
+        #
+        if self.reader.CrystalPermittivity:
+            self.CrystalPermittivity = self.reader.CrystalPermittivity
+        else:
+            # Otherwise create one from the data that has been read in
+            drude = False
+            drude_plasma_au = 0
+            drude_sigma_au = 0
+            sigmas_au = np.array(self.sigmas_cm1)*wavenumber
+            self.CrystalPermittivity = DielectricFunction('dft',parameters=(
+                                         mode_list, frequencies_au, sigmas_au, self.oscillator_strengths,
+                                         volume_au, drude, drude_plasma_au, drude_sigma_au) )
+        # Add the optical permittivity to the dielctric function
+        self.CrystalPermittivity.setEpsilonInfinity(epsilon_inf)
+        #
+        # Prepare to finish
+        #
         self.output_tw.setRowCount(len(self.sigmas_cm1))
         self.output_tw.setColumnCount(5)
         self.output_tw.setHorizontalHeaderLabels(['   Sigma   \n(cm-1)', ' Frequency \n(cm-1)', '  Intensity  \n(Debye2/Å2/amu)', 'Integrated Molar Absorption\n(L/mole/cm2)', 'Absorption maximum\n(L/mole/cm)'])
@@ -243,22 +272,30 @@ class SettingsTab(QWidget):
         self.redraw_output_tw()
         QCoreApplication.processEvents()
         # if self.notebook.spreadsheet is not None:
-        #     self.write_spreadsheet()
-        self.dirty = False
-        QApplication.restoreOverrideCursor()
+        #     self.writeSpreadsheet()
         QCoreApplication.processEvents()
+        debugger.print('Finished:: createIntensityTable')
+        return
 
-    def write_spreadsheet(self):
+    def requestRefresh(self):
+        debugger.print('Start:: requestRefresh')
+        self.refreshRequired = True
+        # self.refresh()
+        debugger.print('Finished:: requestRefresh')
+        return
+
+    def writeSpreadsheet(self):
+        debugger.print('Start:: writeSpreadsheet')
         sp = self.notebook.spreadsheet
         if sp is None:
-            debugger.print('Aborting write of spreadsheet')
+            debugger.print('Finished:: writeSpreadsheet - Aborting write of spreadsheet')
             return
         debugger.print('Writing of spreadsheet')
         sp.selectWorkSheet('Settings')
         sp.delete
         sp.writeNextRow(['Settings and calculations of frequencies and absorption'], row=0, col=1)
         for item in sorted(self.settings):
-            if item == 'Optical permittivity':
+            if item == 'Optical permittivity' and self.settings[item] is not None:
                sp.writeNextRow([item], col=1)
                sp.writeNextRow(self.settings[item][0], col=2, check=1)
                sp.writeNextRow(self.settings[item][1], col=2, check=1)
@@ -274,6 +311,10 @@ class SettingsTab(QWidget):
                sp.writeNextRow(eps[0], col=2, check=1)
                sp.writeNextRow(eps[1], col=2, check=1)
                sp.writeNextRow(eps[2], col=2, check=1)
+            elif item == 'Mass definition':
+               sp.writeNextRow([item,self.settings[item]], col=1, check=1)
+               for c in self.masses_dictionary:
+                   sp.writeNextRow([c,self.masses_dictionary[c]],col=3,check=1)
             else:
                sp.writeNextRow([item,self.settings[item]], col=1, check=1)
         sp.writeNextRow([''], col=1)
@@ -283,11 +324,11 @@ class SettingsTab(QWidget):
             if selected:
                 yn = 'Yes'
             sp.writeNextRow([mode, yn, sigma, f, intensity, 4225.6*intensity, 2*4225.6*intensity/sigma/PI], col=1)
+        debugger.print('Finished:: writeSpreadsheet')
 
     def redraw_output_tw(self):
+        debugger.print('Start:: redraw_output_tw')
         # If the frequencies haven't been set yet just don't try to do anything
-        if not self.frequencies_cm1:
-            return
         self.output_tw.blockSignals(True)
         for i,(f,sigma,intensity) in enumerate(zip(self.frequencies_cm1, self.sigmas_cm1, self.intensities)):
             # Sigma and check / unchecked column
@@ -318,38 +359,39 @@ class SettingsTab(QWidget):
             itemFlags.append( otherFlags )
             for j,(item,flag) in enumerate(zip(items,itemFlags)):
                 item.setFlags(flag)
-                item.setTextAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
+                item.setTextAlignment(int(Qt.AlignHCenter | Qt.AlignVCenter))
                 self.output_tw.setItem(i, j, item )
         # Release the block on signals for the frequency output table
         self.output_tw.resizeColumnsToContents()
         self.output_tw.blockSignals(False)
         QCoreApplication.processEvents()
+        debugger.print('Finished:: redraw_output_tw')
 
     def on_sigma_changed(self):
+        debugger.print('Start:: redraw_output_tw')
         self.settings['Sigma value'] = self.sigma_sb.value()
         self.sigmas_cm1 = [ self.settings['Sigma value'] for i in self.frequencies_cm1 ]
         self.redraw_output_tw()
-        self.notebook.plottingCalculationRequired = True
-        self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
         debugger.print('on sigma change ', self.settings['Sigma value'])
+        self.refreshRequired = True
+        self.refresh()
+        debugger.print('Finished:: on_sigma_changed')
 
     def on_mass_cb_activated(self,index):
-        debugger.print('on mass combobox activated', self.mass_cb.currentText())
+        debugger.print('Start:: on_mass_combobox_activated', self.mass_cb.currentText())
         self.settings['Mass definition'] = self.mass_definition_options[index]
         self.current_mass_definition_index = index
         if index < 3:
             self.mass_cb.model().item(3).setEnabled(False)
         # Modify the element masses
         self.set_masses_tw()
-        self.calculateButtonClicked()
-        self.notebook.plottingCalculationRequired = True
-        self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
+        self.refreshRequired = True
+        self.refresh()
         QCoreApplication.processEvents()
+        debugger.print('Finished:: on_mass_combobox_activated', self.mass_cb.currentText())
 
     def set_masses_tw(self):
-        debugger.print('set masses')
+        debugger.print('Start:: set_masses_tw')
         if self.reader:
             self.element_masses_tw.blockSignals(True)
             species = self.reader.getSpecies()
@@ -391,38 +433,36 @@ class SettingsTab(QWidget):
                 if self.settings['Mass definition'] == 'program':
                     self.element_masses_tw.blockSignals(True)
                     qw.setText('{0:.6f}'.format(mass))
-                    qw.setTextAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
+                    qw.setTextAlignment(int(Qt.AlignHCenter | Qt.AlignVCenter))
                     self.element_masses_tw.setItem(0,i, qw )
                 elif self.settings['Mass definition'] == 'average':
                     self.element_masses_tw.blockSignals(True)
                     qw.setText('{0:.6f}'.format(average_masses[element]))
-                    qw.setTextAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
+                    qw.setTextAlignment(int(Qt.AlignHCenter | Qt.AlignVCenter))
                     debugger.print('average',average_masses[element])
                     self.element_masses_tw.setItem(0,i, qw )
                 elif  self.settings['Mass definition'] == 'isotope':
                     self.element_masses_tw.blockSignals(True)
                     qw.setText('{0:.6f}'.format(isotope_masses[element]))
-                    qw.setTextAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
+                    qw.setTextAlignment(int(Qt.AlignHCenter | Qt.AlignVCenter))
                     debugger.print('isotope',isotope_masses[element])
                     self.element_masses_tw.setItem(0,i, qw )
                 elif  self.settings['Mass definition'] == 'gui':
                     self.element_masses_tw.blockSignals(True)
                     qw.setText('{0:.6f}'.format(self.masses_dictionary[element]))
-                    qw.setTextAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
+                    qw.setTextAlignment(int(Qt.AlignHCenter | Qt.AlignVCenter))
                     debugger.print('gui',self.masses_dictionary[element])
                     self.element_masses_tw.setItem(0,i, qw )
                 else:
                     print('Mass definition not processed', self.settings['Mass definition'])
             # unblock the table signals
             self.element_masses_tw.blockSignals(False)
-            self.notebook.plottingCalculationRequired = True
-            self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
         QCoreApplication.processEvents()
+        debugger.print('Finished:: set_masses_tw')
 
     def on_output_tw_itemChanged(self, item):
         self.output_tw.blockSignals(True)
-        debugger.print('on_output_tw_itemChanged)', item.row(), item.column() )
+        debugger.print('Start:: on_output_tw_itemChanged)', item.row(), item.column() )
         col = item.column()
         row = item.row()
         if col == 0:
@@ -439,54 +479,53 @@ class SettingsTab(QWidget):
             self.redraw_output_tw()
         else:
             self.redraw_output_tw()
-        self.notebook.plottingCalculationRequired = True
-        self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
         QCoreApplication.processEvents()
+        debugger.print('Finished:: on_output_tw_itemChanged')
 
     def on_element_masses_tw_itemClicked(self, item):
-        debugger.print('on_element_masses_tw_itemClicked)', item.row(),item.column() )
+        debugger.print('Start:: on_element_masses_tw_itemClicked)', item.row(),item.column() )
         self.element_masses_tw.blockSignals(False)
+        debugger.print('Finished:: on_element_masses_tw_itemClicked)' )
 
     def on_element_masses_tw_itemChanged(self, item):
-        debugger.print('on_element_masses_tw_itemChanged)', item.row(), item.column() )
+        debugger.print('Start:: on_element_masses_tw_itemChanged)', item.row(), item.column() )
         elements = self.reader.getSpecies()
         col = item.column()
         self.mass_cb.model().item(3).setEnabled(True)
         self.settings['Mass definition'] = 'gui'
         self.mass_cb.setCurrentIndex(3)
         self.masses_dictionary[elements[col]] = float(item.text())
-        self.calculateButtonClicked()
-        self.notebook.plottingCalculationRequired = True
-        self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
         debugger.print('masses_dictionary', self.masses_dictionary)
+        self.refreshRequired = True
+        self.refresh()
+        debugger.print('Finished:: on_element_masses_tw_itemChanged)' )
 
     def on_optical_tw_itemChanged(self, item):
-        debugger.print('on_optical_itemChanged)', item.row(), item.column(), item.text() )
+        debugger.print('Start::on_optical_itemChanged)', item.row(), item.column(), item.text() )
         self.settings['Optical permittivity'][item.row()][item.column()] = float(item.text())
         self.settings['Optical permittivity'][item.column()][item.row()] = float(item.text())
         self.settings['Optical permittivity edited'] = True
-        self.notebook.plottingCalculationRequired = True
-        self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
         self.refresh_optical_permittivity_tw()
+        self.refreshRequired = True
+        self.refresh()
         QCoreApplication.processEvents()
+        debugger.print('Finished::on_optical_itemChanged)')
+        return
 
     def on_optical_tw_itemClicked(self, item):
-        debugger.print('on_optical_itemClicked)', item.row(), item.column() )
+        debugger.print('Start:: on_optical_itemClicked)', item.row(), item.column() )
         self.optical_tw.blockSignals(False)
         QCoreApplication.processEvents()
+        debugger.print('Finished:: on_optical_itemClicked)')
+        return
 
     def refresh(self, force=False):
-        # Refresh the widgets that depend on the reader
+        debugger.print('Start:: refresh', force )
         if not self.reader and self.notebook.reader:
-            self.dirty = True
-        if not self.dirty and not force:
-            debugger.print('Aborting refresh readers are: ',self.reader,self.notebook.reader)
-            debugger.print('Aborting self.dirty and force ',self.dirty,  force)
+            self.refreshRequired = True
+        if not self.refreshRequired:
+            debugger.print('Finished:: refresh not required',force)
             return
-        debugger.print('refresh ',force)
         #
         # Block signals during refresh
         #
@@ -515,42 +554,95 @@ class SettingsTab(QWidget):
         #
         for w in self.findChildren(QWidget):
             w.blockSignals(False)
-        self.calculateButtonClicked()
+        self.createIntensityTable()
+        self.refreshRequired = False
+        # Refresh the scenarios that depend on this dielectric
+        if self.notebook.scenarios is not None:
+            for scenario in self.notebook.scenarios:
+                scenario.requestRefresh()
+        if self.notebook.plottingTab is not None:
+            self.notebook.plottingTab.requestRefresh()
+        if self.notebook.analysisTab is not None:
+            self.notebook.analysisTab.requestRefresh()
+        if self.notebook.fitterTab is not None:
+            self.notebook.fitterTab.requestRefresh()
         QCoreApplication.processEvents()
+        self.refreshRequired = False
+        self.calculationRequired = True
+        debugger.print('Finished:: refresh', force )
+        return
 
     def refresh_optical_permittivity_tw(self):
+        debugger.print('Start:: refresh_optical_permittivity')
         optical = self.settings['Optical permittivity']
         self.optical_tw.blockSignals(True)
         for i,row in enumerate(optical):
             for j, value in enumerate(row):
                 qw = QTableWidgetItem('{0:.4f}'.format(value) )
-                qw.setTextAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
+                qw.setTextAlignment(int(Qt.AlignHCenter | Qt.AlignVCenter))
                 self.optical_tw.setItem(i,j,qw)
         self.optical_tw.blockSignals(False)
         QCoreApplication.processEvents()
+        debugger.print('Finished:: refresh_optical_permittivity')
+        return
 
     def set_optical_permittivity_tw(self):
+        debugger.print('Start:: set_optical_permittivity_tw')
         self.settings['Optical permittivity'] = self.reader.zerof_optical_dielectric
         self.refresh_optical_permittivity_tw()
+        self.refreshRequired = True
         QCoreApplication.processEvents()
+        debugger.print('Finished:: set_optical_permittivity_tw')
+        return
 
     def on_born_changed(self):
-        debugger.print('on born change ', self.born_cb.isChecked())
+        debugger.print('Start:: on_born_change ', self.born_cb.isChecked())
         self.settings['Neutral Born charges'] = self.born_cb.isChecked()
         debugger.print('on born change ', self.settings['Neutral Born charges'])
-        self.calculateButtonClicked()
-        self.notebook.plottingCalculationRequired = True
-        self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
+        self.refreshRequired = True
+        self.refresh()
         QCoreApplication.processEvents()
+        debugger.print('Finished:: on_born_change ', self.born_cb.isChecked())
+        return
 
     def on_eckart_changed(self):
-        debugger.print('on eckart change ', self.eckart_cb.isChecked())
+        debugger.print('Start:: on_eckart_change ', self.eckart_cb.isChecked())
         self.settings['Eckart flag'] = self.eckart_cb.isChecked()
         debugger.print('on eckart change ', self.settings['Eckart flag'])
-        self.calculateButtonClicked()
-        self.notebook.plottingCalculationRequired = True
-        self.notebook.analysisCalculationRequired = True
-        self.notebook.fittingCalculationRequired = True
+        self.refreshRequired = True
+        self.refresh()
         QCoreApplication.processEvents()
+        debugger.print('Finished:: on_eckart_change ', self.eckart_cb.isChecked())
+        return
+
+    def calculate(self,vs_cm1):
+        """Calculate the permittivity of the crystal over the range of frequencies in vs_cm1"""
+        debugger.print('Start:: calculate ')
+        if len(vs_cm1) == 0:
+            debugger.print('Finished:: calculate aborted vs_cm1 has not been set')
+            return
+        self.vs_cm1 = vs_cm1.copy()
+        self.CrystalPermittivity.setUnits('cm-1')
+        dielectricFunction = self.CrystalPermittivity.function()
+        # 
+        self.crystal_permittivity = []
+        debugger.print('About to calculate settings crystal dielectric using pool')
+        # Loop over the frequencies and calculate the crystal dielectric for each frequency
+        for dielecv in self.notebook.pool.imap(dielectricFunction, vs_cm1, chunksize=20):
+            self.crystal_permittivity.append(dielecv)
+            self.notebook.progressbars_update()
+        QCoreApplication.processEvents()
+        self.calculationRequired = False
+        debugger.print('Finished:: calculate ')
+        return
+
+    def get_crystal_permittivity(self,vs_cm1):
+        """Return the crystal permittivity"""
+        debugger.print('Start:: get_crystal_permittivity', self.refreshRequired)
+        if self.calculationRequired or self.refreshRequired or  ( len(self.vs_cm1) != len(vs_cm1) ) or ( self.vs_cm1[0] != vs_cm1[0] ) or ( self.vs_cm1[1] != vs_cm1[1] ) :
+            debugger.print('get_crystal_permittivity refreshing and recalculating' )
+            self.refresh()
+            self.calculate(vs_cm1)
+        debugger.print('Finished:: get_crystal_permittivity', self.refreshRequired)
+        return self.crystal_permittivity
 
